/*
   When we compare this program with struct_2.c
   we see how the C language treats function
   parameters that are arrays differently from
   function parameters that are structs.

   Array parameters to functions always decay to
   a pointer. For array parameters, the "value
   declaration" and the "pointer declaration"
   are equivalent and they mean "pass by pointer".
*/
#include <stdlib.h>
#include <stdio.h>

int f(int arr[]) // the arr parameter decays to a pointer
{
   return 2 + arr[0];
 //return 2 + *arr;   // even though arr is an array, we can use pointer notation
}

int g(int * arr)
{
   return 2 + arr[0]; // even though arr is a pointer, we can use array notation
 //return 2 + *arr;
}

int main()
{
   int a1[5];                          // a1 is declared as an array value
   int * a2 = malloc(5 * sizeof(int)); // a2 is declared as an array pointer
   a1[1] = 3;
   a2[1] = 3;      // hide the fact that a2 is a pointer
   *(a1 + 2) = 4;  // show that a1 decays to a pointer
   *(a2 + 2) = 4;

   a1[0] = -1;
   a2[0] = -1;

   int x1 = f(a1); // call f with an array value argument
   int x2 = f(a2); // call f with an array pointer argument

   int y1 = g(a1); // call g with an array value argument
   int y2 = g(a2); // call g with an array pointer argument

   printf("%d %d %d %d %d\n", a1[0], a1[1], a1[2], a1[3], a1[4]);
   printf("%d %d %d %d %d\n", a2[0], a2[1], a2[2], a2[3], a2[4]);
   printf("%d %d\n", x1, x2);
   printf("%d %d\n", y1, y2);

   free(a2);

   return 0;
}
