/*
   When we compare this program with struct_3.c
   we see how the C language treats function
   parameters that are arrays differently from
   function parameters that are structs.

   Array parameters to functions are always IN/OUT
   parameters (unless they are declared as const).
   This means that array parameters can be used for
   "side effects". See
      https://en.wikipedia.org/wiki/Side_effect_(computer_science)
*/
#include <stdlib.h>
#include <stdio.h>

void f(int arr[], int n) // n is the length of the array
{
   arr[n-1] = arr[0] + 1;  // this is a "side effect"
}

void g(int * arr, int n) // n is the length of the "array"
{
   arr[n-1] = arr[0] + 1;  // this is a "side effect"
}

int main()
{
   int a1[5];                          // a1 is declared as an array value
   int * a2 = malloc(5 * sizeof(int)); // a2 is declared as an array pointer
   a1[1] = 3;
   a2[1] = 3;      // hide the fact that a2 is a pointer
   *(a1 + 2) = 4;  // show that a1 decays to a pointer
   *(a2 + 2) = 4;

   a1[0] = -1;
   a2[0] = -1;

   printf("%d %d %d %d %d\n", a1[0], a1[1], a1[2], a1[3], a1[4]);
   printf("%d %d %d %d %d\n", a2[0], a2[1], a2[2], a2[3], a2[4]);

   f(a1, 5); // call f with an array value argument
   f(a2, 5); // call f with an array pointer argument

   printf("%d %d %d %d %d\n", a1[0], a1[1], a1[2], a1[3], a1[4]);
   printf("%d %d %d %d %d\n", a2[0], a2[1], a2[2], a2[3], a2[4]);

   g(a1, 4); // call g with an array value argument
   g(a2, 4); // call g with an array pointer argument

   printf("%d %d %d %d %d\n", a1[0], a1[1], a1[2], a1[3], a1[4]);
   printf("%d %d %d %d %d\n", a2[0], a2[1], a2[2], a2[3], a2[4]);

   free(a2);

   return 0;
}
