/*
   When we compare this program with array_2.c
   we see how the C language treats function
   parameters that are arrays differently from
   function parameters that are structs.

   For struct parameters, the "value declaration" means
   the parameter is "passed by value" (i.e., copied).
   The "pointer declaration" means the parameter is
   "passed by pointer" (as array parameters always are).
*/
#include <stdlib.h>
#include <stdio.h>

struct S {
   int n;
   double x;
};

int f(struct S s) // the parameter s is copied(!) to the function
{
   return 2 + s.n;
}

int g(struct S * s)
{
   return 2 + s->n;
}

int main()
{
   struct S s1 = {4, 5.6};                   // s1 is declared as a struct value
   struct S * s2 = malloc(sizeof(struct S)); // s2 is declared as a struct pointer
   s1.x = 7.8;
   s2->x = 7.8;
   (*s2).x = 8.9;
   (*s2).n = 6;

   int x1 = f(s1);
   int x2 = f(*s2);

   int y1 = g(&s1);
   int y2 = g(s2);

   printf("%d %f %d %f\n", s1.n, s1.x, s2->n, s2->x);
   printf("%d %d\n", x1, x2);
   printf("%d %d\n", y1, y2);

   free(s2);

   return 0;
}
