/*
   When we compare this program with array_3.c
   we see how the C language treats function
   parameters that are arrays differently from
   function parameters that are structs.

   For struct parameters, a "value declaration"
   means the parameter is strictly an IN parameter
   (no side effects). A "pointer declaration" means
   the parameter is an IN/OUT parameter (unless it's
   declared as const).
*/
#include <stdlib.h>
#include <stdio.h>

struct S {
   int n;
   double x;
};

void f(struct S s) // the parameter s is copied(!) to the function
{
   s.x = s.x + s.n;  // what does this do?
}

void g(struct S * s)
{
   s->x = s->x + s->n;  // this is a "side effect"
}

int main()
{
   struct S s1 = {4, 5.6};                   // s1 is declared as a struct value
   struct S * s2 = malloc(sizeof(struct S)); // s2 is declared as a struct pointer
   s1.x = 7.8;
   s2->x = 7.8;
   (*s2).x = 8.9;
   (*s2).n = 6;

   printf("%d %f %d %f\n", s1.n, s1.x, s2->n, s2->x);

   f(s1);   // call f with a struct value argument
   f(*s2);  // call f with a struct value argument

   printf("%d %f %d %f\n", s1.n, s1.x, s2->n, s2->x);

   g(&s1);  // call g with a pointer to a struct value
   g(s2);   // call g with a pointer to a struct value

   printf("%d %f %d %f\n", s1.n, s1.x, s2->n, s2->x);

   free(s2);

   return 0;
}
